@@ -3,13 +3,21 @@ pgmm <- function(formula, data, subset, na.action,
                  model = c("onestep", "twosteps"),
                  transformation = c("d", "ld"), fsm = NULL,
                  index = NULL, ...){
+
+  # yX : response / covariates, W : gmm instruments, Z : normal
+  # instruments, V : time dummies
+  
   cl <- match.call(expand.dots = FALSE)
   effect <- match.arg(effect)
   model <- match.arg(model)
   transformation <- match.arg(transformation)
+  namesV <- NULL
+  
+  #################################################################
+  ##### 1. Backward compatibility with the old formula / dynformula
+  ##### interface
+  #################################################################
   
-  # the following section ensures backward compatibility with the old
-  # formula and dynformula interfaces
   if (inherits(formula, "dynformula") || length(Formula(formula))[2] == 1){
     if (!inherits(formula, "dynformula")){
       formula <- match.call(expand.dots = TRUE)
@@ -38,22 +46,32 @@ pgmm <- function(formula, data, subset, na.action,
     gmm.form <- dynterms2formula(gmm.lags)
     formula <- as.Formula(main.form, gmm.form)
   }
+
+  #################################################################
+  ##### 2. Extract the response/covariates, the gmm instruments and
+  ##### the "normal" instruments, as a named list containing the lag
+  ##### structure
+  #################################################################
   
   x <- formula
   if (!inherits(x, "Formula")) x <- Formula(formula)
   # gmm instruments : named list with the lags, names being the variables
   gmm.form <- formula(x, rhs = 2, lhs = 0)
   gmm.lags <- dynterms(gmm.form)
+
   # covariates : named list with the lags, names being the variables
   main.form <- formula(x, rhs = 1, lhs = 1)
   main.lags <- dynterms(main.form)
+
   # Three possibilities for 'normal' instruments :
   # 1. the third part of the formula describes them
   # 2. all variables not used as gmm are normal instruments
   # 3. all variables are gmm instruments and therefore, there are no
   # normal instruments except maybe time dummies
+  
   # the third part of the formula (if any) deals with the 'normal' instruments
   if (length(x)[2] == 3){
+    normal.instruments <- TRUE
     inst.form <- formula(x, rhs = 3, lhs = 0)
     # the . - x1 + x2 syntax is allowed, in this case update with the first part
     inst.form <- update(main.form, inst.form)
@@ -66,14 +84,22 @@ pgmm <- function(formula, data, subset, na.action,
     iv <- names(main.lags)[! names(main.lags) %in% names(gmm.lags)]
     inst.lags <- main.lags[iv]
     # generate the formula for 'normal' instruments
-    if (length(inst.lags) > 0) inst.form <- dynterms2formula(inst.lags)
+    if (length(inst.lags) > 0){
+      normal.instruments <- TRUE
+      inst.form <- dynterms2formula(inst.lags)
+    }
     else{
       # the case where there are no normal instruments : put inst.form
       # and inst.lags to NULL
+      normal.instruments <- FALSE
       inst.form <- NULL
       inst.lags <- NULL
     }
   }
+
+  #################################################################
+  ##### 3. How many time series are lost
+  #################################################################
   
   # How many time series are lost ? May be the maximum number of lags
   # of any covariates + 1 because of first - differencing or the
@@ -84,218 +110,262 @@ pgmm <- function(formula, data, subset, na.action,
   if (!is.null(inst.lags)) inst.maxlag <- max(sapply(inst.lags, max))
   else inst.maxlag <- 0
   main.maxlag <- max(sapply(main.lags, max))
-  time.lost <- max(main.maxlag + 1, inst.maxlag + 1, gmm.minlag)
-  time.lost.level <- max(main.maxlag, inst.maxlag)
-  # if time.lost.level = 0 (no lags), one observation is lost anyway
-  # because of the differentiation of the lag instruments
-  time.lost.level <- max(1, time.lost.level)
-  # Compute the model frame using the covariates, the 'normal'
-  # instruments if any and the gmm instruments without the lags
+  TL1 <- max(main.maxlag + 1, inst.maxlag + 1, gmm.minlag)
+  TL2 <- max(main.maxlag, inst.maxlag)
+  TL2 <- max(main.maxlag, inst.maxlag, gmm.minlag - 1)
+  # if TL2 = 0 (no lags), one observation is lost anyway because of
+  # the differentiation of the lag instruments
+  TL2 <- max(1, TL2)
+
+  #################################################################
+  ##### 4. Compute the model frame which contains the
+  ##### response/covariates, the gmm instruments and the 'normal'
+  ##### instruments without the lags
+  #################################################################
+  
   gmm.form <- as.formula(paste("~", paste(names(gmm.lags), collapse = "+")))
-  if (!is.null(inst.form))  F <- as.Formula(main.form, gmm.form, inst.form)
-  else F <- as.Formula(main.form, gmm.form)
+  if (!is.null(inst.form))  Form <- as.Formula(main.form, gmm.form, inst.form)
+  else Form <- as.Formula(main.form, gmm.form)
   mf <- match.call(expand.dots = FALSE)
-  m <- match(c("formula", "data", "subset", "na.action", "index"),names(mf),0)
-  mf <- mf[c(1,m)]
+  m <- match(c("formula", "data", "subset", "na.action", "index"), names(mf), 0)
+  mf <- mf[c(1, m)]
   mf$drop.unused.levels <- TRUE
   mf[[1]] <- as.name("plm")
   mf$model <- NA
-  mf$formula <- F
+  mf$formula <- Form
   mf$na.action <- "na.pass"
   data <- eval(mf, parent.frame())
-  index <- attr(data, "index")
+  index <- index(data)
+  N <- length(levels(index[[1]]))
   T <- length(levels(index[[2]]))
   pdim <- pdim(data)
+  balanced <- pdim$balanced
 
-  # Compute the first step matrices
-  if (transformation == "d") A1 <- tcrossprod(diff(diag(1, T - time.lost + 1)))
-  if (transformation == "ld") A1 <- FSM(T - time.lost.level, "full")
+  # if the data is unbalanced, extract the number of missing time
+  # series at the begining and at the end and "balance" the data
+  if (!balanced){
+    nats <- tapply(as.numeric(as.factor(index(data, "time"))),
+                   index(data, "id"),
+                   function(x) c(- 1, T) + c(1, - 1) * range(x))
+    un.id <- sort(unique(index(data, "id")))
+    un.time <- sort(unique(index(data, "time")))
+    rownames(data) <- paste(index(data, "id"), index(data, "time"), sep = ".")
+    allRows <- as.character(t(outer(un.id, un.time, paste, sep = ".")))
+    data <- data[allRows, ]
+    rownames(data) <- allRows
+    index <- data.frame(id = rep(un.id, each = length(un.time)),
+                        time = rep(un.time, length(un.id)),
+                        row.names = rownames(data))
+    class(index) <- c("pindex", "data.frame")
+    attr(data, "index") <- index
+  }
   
-  # Get the covariates matrix, split it by individual 
+  #################################################################
+  ##### 5. Get the response/covariates matrix yX, the gmm instruments
+  ##### matrix W and the normal instruments matrix inst, splited by
+  ##### individuals
+  #################################################################
+
   attr(data, "formula") <- formula(main.form)
   yX <- extract.data(data)
   names.coef <- colnames(yX[[1]])[-1]
-  # Get a list of missing time series for each individual : nats is a
-  # list of two dimentional vectors containing the number of time
-  # series lost at the begining and at the end of the series
-  rn <- lapply(yX, rownames)
-  allrn <- levels(attr(data, "index")[[2]])
-  nats <- lapply(rn,
-               function(x){
-                 z <- allrn %in% x
-                 z <- which(z)
-                 c(z[1] - 1, T - z[length(z)])
-               }
-               )
-  # Get the instruments and GMM matrix, split them by individual
-  if (!is.null(inst.form)){
+  if (normal.instruments){
     attr(data, "formula") <- inst.form
-    inst <- extract.data(data)
+    Z <- extract.data(data)
   }
-  else inst <- NULL
-
+  else Z <- NULL
   attr(data, "formula") <- gmm.form
   W <- extract.data(data, as.matrix = FALSE)
-  # Add an nats attribute to the three lists of matrices
-  W <- mapply(function(x, y){ attr(x, "nats") <- y;x}, W, nats, SIMPLIFY=FALSE)
-  if (!is.null(inst))
-    inst <- mapply(function(x, y){ attr(x, "nats") <- y;x}, inst, nats, SIMPLIFY=FALSE)
-  yX <- mapply(function(x, y){ attr(x, "nats") <- y;x}, yX, nats, SIMPLIFY=FALSE)
-  # Create the matrix of time dummies.
-  namest <- levels(attr(data, "index")[,2])
+  
+  #################################################################
+  ##### 6. Create the matrix of response/covariates, gmm instruments
+  ##### and normal instruments for the diff model
+  #################################################################
+
+  makegmm <- function(x, g){
+    rg <- range(g)
+    # Create the relevant matrix for one gmm instrument
+    z <- as.list((TL1 + 1):T)
+    x <- lapply(z, function(y) x[max(1, y - rg[2]):(y - rg[1])])
+    lx <- sapply(x, length)
+    n <- length(x)
+    lxc <- cumsum(lx)
+    before <- c(0, lxc[-n])
+    after <- lxc[n] - sapply(x, length) - before
+    result <- mapply(function(x, y, z)
+                     c(rep(0, y), x, rep(0, z)),
+                     x, before, after, SIMPLIFY = TRUE)
+    t(result)
+  }
+  # create the matrix of gmm instruments for every individual
+  W1 <- lapply(W,
+               function(x){
+                 u <- mapply(makegmm, x, gmm.lags, SIMPLIFY = FALSE)
+                 u <- matrix(unlist(u), nrow = nrow(u[[1]]))
+                 u
+               }
+               )
+  # differenciate the matrix of response/covariates (and of normal
+  # instruments if any) and remove T1 - 1 time series (xd is already
+  # differenced)
+  yX1 <- lapply(yX,
+                function(x){
+                  xd <- diff(x)
+                  xd <- xd[- c(1:(TL1 - 1)), , drop = FALSE]
+                  xd
+                }
+                )
+  if (normal.instruments){
+    Z1 <- lapply(Z,
+                 function(x){
+                   xd <- diff(x)
+                   xd <- xd[- c(1:(TL1 - 1)), , drop = FALSE]
+                   xd
+                 }
+                 )
+  }
+
+  #################################################################
+  ##### 7. In case of system gmm, create the matrix of
+  ##### response/covariates, gmm instruments and normal instruments
+  ##### for the level model and merge with the diff model
+  #################################################################
+
+  if (transformation == "ld"){
+    W2 <- lapply(W,
+                 function(x){
+                   u <- lapply(x, function(z) diag(diff(z[-c(length(z))])))
+                   # the matrix of instruments in difference has T - 2
+                   # rows if one time series is lost (there are no gmm
+                   # instruments for t = 2 but there is a moment
+                   # condition with the intercept. In this case, a row
+                   # of 0 should be added. Otherwise, the number of
+                   # rows is just T - TL2
+                   nrow.ud <- ifelse(TL2 == 1, T - 2, T - TL2)
+                   u <- matrix(unlist(u), nrow = nrow.ud)
+                   if (TL2 == 1) u <- rbind(0, u)
+                   u
+                 }
+                 )
+    # remove the relevent number of time series for data in level
+    yX2 <- lapply(yX,
+                  function(x){
+                    x <- x[- c(0:TL2), , drop = FALSE]
+                    x
+                  }
+                  )
+    if (normal.instruments){
+      Z2 <- lapply(Z, function(x){x <- x[- c(0:TL2), , drop = FALSE]; x})
+    }
+  }
+
+  #################################################################
+  ##### 8. Add time dummies if effect = twoways
+  #################################################################
+
   if (effect == "twoways"){
-    if (transformation == "ld"){
+    namesV <- levels(index(data, which = "time"))
+    if (transformation == "d"){
+      V1 <-td.model.diff <- diff(diag(1, T - TL1 + 1))[, -1]
+      namesV <- namesV[- c(0:(TL1))]
+    }
+    else{
       td <- cbind(1, rbind(0, diag(1, T - 1)))
       # remove as many columns and row as there are lost time series
       # in level (the difference of position between rows and columns
       # is due to the fact that the first column of td is the
       # intercept and should be kept anyway
-      td.gmm.level <- td.model.level <-
-        td[-c(1:time.lost.level), - c(2:(2+time.lost.level-1))]
-      td.model.diff <- diff(td.model.level)
-      namest <- c("(intercept)", namest[-c(0:time.lost.level+1)])
+      V2 <- td[- c(1:TL2), - c(2:(2 + TL2 - 1))]
+      V1 <- diff(V2)
+      namesV <- c("(intercept)", namesV[- c(0:TL2 + 1)])
     }
-    if (transformation == "d"){
-      td.gmm.diff <- td.model.diff <- diff(diag(1, T - time.lost + 1))[, -1]
-      namest <- namest[-c(0:(time.lost))]
+    for (i in 1:N){
+      yX1[[i]] <- cbind(yX1[[i]], V1)
+      if (transformation == "d"){
+        W1[[i]] <- cbind(W1[[i]], V1)
+      }
+      else{
+        W2[[i]] <- cbind(W2[[i]], V2)
+        yX2[[i]] <- cbind(yX2[[i]], V2)
+      }
     }
   }
   # A QAD fix for the bug in mtest for ld model without time.dummies
-  if (effect == "individual" && transformation == "ld")
-    namest <- c("(intercept)", namest[-c(0:time.lost.level+1)])
-
-  # create the matrix of gmm instruments for every individual
-  W <- lapply(W,
-              # loop on all the individuals
-              function(x){
-                nats <- attr(x, "nats")
-                # loop on every gmm instrument
-                u <- mapply(
-                            function(x, g){
-                              rg <- range(g)
-                              # Create the relevant matrix for one gmm instrument
-                              x <- c(rep(0, nats[1]), x, rep(0, nats[2]))
-                              z <- as.list((time.lost+1):T)
-                              x <- lapply(z, function(y) x[max(1, y - rg[2]):(y - rg[1])])
-                              lx <- sapply(x, length)
-                              n <- length(x)
-                              lxc <- cumsum(lx)
-                              before <- c(0, lxc[-n])
-                              after <- lxc[n] - sapply(x, length) - before
-                              result <- mapply(function(x, y, z)
-                                               c(rep(0, y), x, rep(0, z)),
-                                               x, before, after)
-                              matrix(unlist(result), nrow = length(x), byrow = TRUE)
-                            },
-                            x, gmm.lags,
-                            SIMPLIFY = FALSE
-                            )
-                # cbind all the matrices and owerwrite rows with 0 if
-                # there are some missing time series
-                nr <- nrow(u[[1]])
-                u <- unlist(u)
-                u <- matrix(u, nrow = nr)
-                # add the matrix of time dummies if required and if
-                # diff gmm is computed
-                if (effect == "twoways"){
-                  if (transformation == "d") u <- cbind(u, td.gmm.diff)
-                }
-                if (nats[1]) u[1:nats[1], ] <- 0
-                if (nats[2]) u[(T - time.lost - nats[2] + 1):(T - time.lost), ] <- 0
-                # in case of 'sys' gmm, add for each time series the
-                # difference, laged 1 of every gmm instrument
-                if (transformation == "ld"){
-                  ud <- lapply(x,
-                               function(z)
-                               diag(c(rep(0, nats[1]), diff(z[-c(length(z))]), rep(0, nats[2])))
-                               )
-                  # the matrix of instruments in difference has T - 2
-                  # rows if one time series is lost (there are no gmm
-                  # instruments for t = 2 but there is a moment
-                  # condition with the intercept. In this case, a row
-                  # of 0 should be added. Otherwise, the number of
-                  # rows is just T - time.lost.level
-                  nrow.ud <- ifelse(time.lost.level == 1, T - 2, T - time.lost.level)
-                  ud <- matrix(unlist(ud), nrow = nrow.ud)
-                  if (time.lost.level == 1) ud <- rbind(0, ud)
-                  if (effect == "twoways") ud <- cbind(ud, td.gmm.level)
-                  # owerwrite rows of  missing time series with 0
-                  if (nats[1]) ud[c(0:nats[1]), ] <- 0
-                  if (nats[2]) ud[(T-time.lost.level- nats[2]+1):(T-time.lost.level), ] <- 0
-                  u <- bdiag(u, ud)
-                }
-                u
-              }
-              )
-
-  yX <- lapply(yX,
-               function(x){
-                 nats <- attr(x, 'nats')
-                 nc <- ncol(x)
-                 # add as many rows of NAs than there are missing
-                 # time series, at the begining and/or at the end
-                 before <- matrix(NA, nats[1] ,nc)
-                 after <- matrix(NA, nats[2], nc)
-                 x <- rbind(before, x, after)
-                 xd <- diff(x)
-                 # remove time.lost - 1 time series (xd is already
-                 # differenced
-                 xd <- xd[-c(1:(time.lost-1)), , drop = FALSE]
-                 # add time dummies if required
-                 if (effect == "twoways") xd <- cbind(xd, td.model.diff)
-                 # put 0 lignes for missing time series 
-                 if (nats[1]) xd[0:nats[1], ] <- 0
-                 if (nats[2]) xd[(T-time.lost- nats[2]+1):(T-time.lost), ] <- 0
-                 if (transformation == "ld"){
-                   # remove the relevent number of time series for
-                   # data in level
-                   x <- x[- c(0:time.lost.level), , drop=FALSE]
-                   # add the matrix of time dummies if required
-                   if (effect == "twoways"){
-                     x <- cbind(x, td.model.level)
-                   }
-                   # owerwrite rows of  missing time series with 0
-                   if (nats[1]) x[c(0:nats[1]), ] <- 0
-                   if (nats[2]) x[(T-time.lost.level- nats[2]+1):(T-time.lost.level), ] <- 0
-                   xd <- rbind(xd, x)
-                 }
-                 xd
-               }
-               )
-
-  if (!is.null(inst)){
-    inst <- lapply(inst,
-                   function(x){
-                     nats <- attr(x, 'nats')
-                     nc <- ncol(x)
-                     # add as many rows of NAs than there are missing
-                     # time series, at the begining and/or at the end
-                     before <- matrix(NA, nats[1] ,nc)
-                     after <- matrix(NA, nats[2], nc)
-                     x <- rbind(before, x, after)
-                     xd <- diff(x)
-                     # remove time.lost - 1 time series (xd is already
-                     # differenced
-                     xd <- xd[-c(1:(time.lost-1)), , drop = FALSE]
-                     # put 0 lignes for missing time series 
-                     if (nats[1]) xd[1:nats[1], ] <- 0
-                     if (nats[2]) xd[(T-time.lost- nats[2]+1):(T-time.lost), ] <- 0
-                     if (transformation == "ld"){
-                       # remove the relevent number of time series for
-                       # data in level
-                       x <- x[- c(0:time.lost.level), , drop=FALSE]
-                       # put 0 lignes for missing time series 
-                       if (nats[1]) x[c(0:nats[1]), ] <- 0
-                       if (nats[2]) x[(T-time.lost.level- nats[2]+1):(T-time.lost.level), ] <- 0
-                       xd <- bdiag(xd, x)
-                     }
-                     xd
-                   }
-                   )
-    # cbind the matrices of gmm and normal instruments
-    W <- mapply(cbind, W, inst, SIMPLIFY = FALSE)
+  if (effect == "individual" && transformation == "ld"){
+    namesV <- levels(index(data, which = "time"))
+    namesV <- c("(intercept)", namesV[-c(0:TL2 + 1)])
   }
+  
+  #################################################################
+  ##### 9. In case of unbalanced data, replace NA's by 0 and overwrite
+  ##### rows for missing time series with 0
+  #################################################################
+
+  if (!balanced){
+    T1 <- T - TL1
+    T2 <- T - TL2
+    for (i in 1:N){
+      nas <- nats[[i]]
+      mb <- nas[1]
+      me <- nas[2]
+      narows <- apply(yX1[[i]], 1, function(z) any(is.na(z)))
+      yX1[[i]][narows, ] <- 0
+      W1[[i]][is.na(W1[[i]])] <- 0
+      if (mb) W1[[i]][c(0:mb), ] <- 0
+      if (me) W1[[i]][(T1 - me + 1):T1, ] <- 0      
+      if (transformation == "ld"){
+        W2[[i]][is.na(W2[[i]])] <- 0
+        yX2[[i]][is.na(yX2[[i]])] <- 0
+        if (mb) W2[[i]][c(0:mb), ] <- yX2[[i]][c(0:mb), ] <- 0
+        if (me) W2[[i]][(T2 - me + 1):T2, ] <- yX2[[i]][(T2 - me + 1):T2, ] <- 0
+      }
+      if (normal.instruments){
+        Z1[[i]][is.na(Z1[[i]])] <- 0
+        if (mb) Z1[[i]][c(0:mb), ] <- 0
+        if (me) Z1[[i]][(T1 - me + 1):T1, ] <- 0      
+        if (transformation == "ld"){
+          Z2[[i]][is.na(Z2[[i]])] <- 0
+          if (mb) Z2[[i]][c(0:mb), ] <- 0
+          if (me) Z2[[i]][(T2 - me + 1):T2, ] <- 0
+        }
+      }
+    }
+  }
+
+  #################################################################
+  ##### 10. In case of sys gmm, biag or rbind the diff and level
+  ##### matrices
+  #################################################################
+  
+  if (transformation == "ld"){
+    for (i in 1:N){
+      W1[[i]] <- bdiag(W1[[i]], W2[[i]])
+      yX1[[i]] <- rbind(yX1[[i]], yX2[[i]])
+      if (normal.instruments) Z1[[i]] <- bdiag(Z1[[i]], Z2[[i]])
+    }
+  }
+  if (normal.instruments){
+    for (i in 1:N) W1[[i]] <- cbind(W1[[i]], Z1[[i]])
+  }
+
+  
+  #################################################################
+  ##### 11. Compute the estimator
+  #################################################################
+
+  W <- W1 ; yX <- yX1
+  
+  # Compute the first step matrices
+  if (transformation == "d") A1 <- tcrossprod(diff(diag(1, T - TL1 + 1)))
+  if (transformation == "ld") A1 <- FSM(T - TL2, "full")
+ 
   # compute the estimator
+  
+  ## WX <- mapply(function(x, y) crossprod(x, y), W, yX, SIMPLIFY = FALSE)
+  ## WX <- Reduce("+", WX)
+  ## zerolines <- which(apply(WX, 1, function(z) sum(abs(z))) == 0)
+  ## for (i in 1:N) W[[i]] <- W[[i]][, - zerolines]
+
   WX <- mapply(function(x, y) crossprod(x, y), W, yX, SIMPLIFY = FALSE)
   Wy <- lapply(WX, function(x) x[, 1])
   WX <- lapply(WX, function(x) x[, -1])
@@ -303,10 +373,11 @@ pgmm <- function(formula, data, subset, na.action,
   A1 <- solve(Reduce("+", A1))*length(W)
   WX <- Reduce("+", WX)
   Wy <- Reduce("+", Wy)
+
   B1 <- solve(crossprod(WX, t(crossprod(WX, A1))))
   Y1 <- crossprod(t(crossprod(WX, A1)), Wy)
   coefficients <- as.numeric(crossprod(B1, Y1))
-  if (effect == "twoways") names.coef <- c(names.coef, namest)
+  if (effect == "twoways") names.coef <- c(names.coef, namesV)
   names(coefficients) <- names.coef
   residuals <- lapply(yX,
                       function(x)
@@ -338,7 +409,7 @@ pgmm <- function(formula, data, subset, na.action,
   fitted.values <- mapply(function(x,y) x[, 1] - y, yX, residuals)
   if (model == "twosteps") coefficients <- list(coef1s, coefficients)
   args <- list(model = model, effect = effect,
-               transformation = transformation, namest = namest)
+               transformation = transformation, namest = namesV)
   result <- list(coefficients = coefficients, residuals = residuals, vcov = vcov,
                  fitted.values = fitted.values,
                  df.residual = df.residual, 
@@ -364,7 +435,6 @@ dynterms <- function(x){
   resu
 }
 
-
 getvar <- function(x){
   x <- as.list(x)
   result <- lapply(x, function(y){
@@ -459,7 +529,6 @@ extract.data <- function(data, as.matrix = TRUE){
   data
 }
 
-
 G <- function(t){
   G <- matrix(0,t,t)
   for (i in 1:(t-1)){
@@ -471,7 +540,6 @@ G <- function(t){
   G
 }
 
-
 FD <- function(t){
   FD <- Id(t)[-1,]
   for (i in 1:(t-1)){
